# Copyright 2019 - China Mobile (SuZhou) Software Technology Co.,Ltd.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.


class HostProperties(object):
    HOST = 'host'
    ZONE = 'zone'


class ZoneProperties(object):
    ZONE_NAME = 'zoneName'
    ZONE_STATE = 'zoneState'
    NOVE_COMPUTE = 'nova-compute'
    ACTIVE = 'active'
