# Copyright 2016 - Nokia
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.


class CinderProperties(object):
    SIZE = 'size'
    VOLUME_TYPE = 'volume_type'
    ATTACHMENTS = 'attachments'
    DISPLAY_NAME = 'display_name'
    ID = 'id'
    STATUS = 'status'
    CREATED_AT = 'created_at'
    VOLUME_ATTACHMENT = 'volume_attachment'
    INSTANCE_UUID = 'instance_uuid'
    SERVER_ID = 'server_id'
    VOLUME_ID = 'volume_id'
    UPDATE_AT = 'updated_at'
    TENANT_ID = 'tenant_id'
