# Copyright 2019 - China Mobile (SuZhou) Software Technology Co.,Ltd.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.


class NetworkProperties(object):
    NAME = 'name'
    STATUS = 'status'
    ID = 'id'
    TENANT_ID = 'tenant_id'
    CREATED_AT = 'created_at'
    UPDATED_AT = 'updated_at'
    NETWORK = 'network'


class PortProperties(object):
    NAME = 'name'
    STATUS = 'status'
    ID = 'id'
    TENANT_ID = 'tenant_id'
    CREATED_AT = 'created_at'
    UPDATED_AT = 'updated_at'
    NETWORK = 'network'
    DEVICE_ID = 'device_id'
    NETWORK_ID = 'network_id'
    HOST_ID = 'host_id'
    PORT = 'port'
    IP_ADDRESS = 'ip_address'
    IP_ADDRESSES = 'ip_addresses'
