# Copyright 2016 - Nokia
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from collections import namedtuple

from vitrage.evaluator.template_fields import TemplateFields
from vitrage.evaluator.template_schema_factory import TemplateSchemaFactory

Template = namedtuple('Template', ['uuid', 'data', 'date'])
TEMPLATE_LOADER = 'template_loader'
SYNTAX = 'syntax'
CONTENT = 'content'


def get_template_schema(template):
    metadata = template.get(TemplateFields.METADATA)
    if metadata is None:
        return None
    version = metadata.get(TemplateFields.VERSION,
                           TemplateSchemaFactory.DEFAULT_VERSION)
    return TemplateSchemaFactory().template_schema(version)
